import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_mobx/flutter_mobx.dart';
import 'package:flutter_modular/flutter_modular.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:vesti_logger/logger/vlog.dart';
import 'package:vesti_logger/logger/vlogger_store.dart';

class VLoggerList extends StatefulWidget {
  @override
  _VLoggerListState createState() => _VLoggerListState();
}

class _VLoggerListState extends State<VLoggerList> {
  VLoggerStore _store = Modular.get();
  final TextEditingController _controller = TextEditingController();

  @override
  void initState() {
    super.initState();
    _store.reset();
    _controller.addListener(() {
      _store.filter = _controller.text;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        _headerWidget(),
        SizedBox(height: 8),
        Expanded(
          child: Observer(
            builder: (_) =>
                ListView.builder(
                    itemCount: _store.filteredLogs.length,
                    itemBuilder: (context, index) {
                      return _logWidget(index);
                    }),
          ),
        ),
      ],
    );
  }

  Widget _headerWidget() {
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 16.0),
      child: Row(
        children: [
          Expanded(
            child: TextField(
              controller: _controller,
              decoration: InputDecoration(hintText: "Pesquisa"),
              onSubmitted: (text) {
                _store.filter = text;
              },
            ),
          ),
          Row(
              children: VLogType.values
                  .map((value) => _typeItemWidget(value))
                  .toList())
        ],
      ),
    );
  }

  Widget _typeItemWidget(VLogType value) {
    return Observer(
      builder: (_) =>
          RippleCard(
            color: _colorTypeItem(value),
            child: Padding(
              padding: const EdgeInsets.all(8.0),
              child: value.icon,
            ),
            onTap: () {
              _onTapTypeItem(value);
            },
          ),
    );
  }

  Color _colorTypeItem(VLogType value) {
    switch (value) {
      case VLogType.Debug:
        return _store.isDebug ? Colors.grey : Colors.transparent;
      case VLogType.Error:
        return _store.isError ? Colors.grey : Colors.transparent;
      case VLogType.Info:
        return _store.isInfo ? Colors.grey : Colors.transparent;
    }
  }

  _onTapTypeItem(VLogType value) {
    switch (value) {
      case VLogType.Debug:
        _store.isDebug = !_store.isDebug;
        break;
      case VLogType.Error:
        _store.isError = !_store.isError;
        break;
      case VLogType.Info:
        _store.isInfo = !_store.isInfo;
        break;
    }
  }

  Widget _logWidget(int index) {
    if (_store.filteredLogs.length > index) {
      var log = _store.filteredLogs[index];
      return Padding(
        padding: const EdgeInsets.symmetric(horizontal: 16, vertical: 8),
        child: RippleCard(
          elevation: 3,
          color: Colors.grey,
          child: Padding(
            padding: const EdgeInsets.all(8.0),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              children: [
                Row(
                  children: [
                    log.type?.icon ?? VLogType.Debug.icon,
                    SizedBox(width: 8),
                    Expanded(
                      child: Text(log.title ?? "",
                          style: Theme
                              .of(context)
                              .textTheme
                              .headline5),
                    )
                  ],
                ),
                Flexible(
                  child: Text(log.message ?? "",
                      style: Theme
                          .of(context)
                          .textTheme
                          .caption
                          ?.copyWith(fontWeight: FontWeight.w700)),
                ),
              ],
            ),
          ),
          onTap: () {
            Clipboard.setData(new ClipboardData(text: log.message));
            Fluttertoast.showToast(
              timeInSecForIosWeb: 4,
              msg: "Copiado para o teclado",
              backgroundColor: Colors.black87,
              gravity: ToastGravity.TOP,
            );
          },
        ),
      );
    }
    return Container();
  }
}


class RippleCard extends StatelessWidget {
  final Widget? child;
  final Color? color;
  final BorderSide side;
  final BorderRadius borderRadius;
  final double? elevation;
  final GestureTapCallback? onTap;

  RippleCard({@required this.child,
    this.onTap,
    this.elevation = 1,
    this.color = Colors.transparent,
    this.side = BorderSide.none,
    this.borderRadius = const BorderRadius.all(Radius.circular(8))});

  @override
  Widget build(BuildContext context) {
    return Card(
        elevation: elevation,
        shape: RoundedRectangleBorder(
          side: side,
          borderRadius: borderRadius,
        ),
        child: ClipRRect(
            borderRadius: borderRadius,
            child: InkWell(
                borderRadius: borderRadius,
                child: Ink(
                    decoration:
                    BoxDecoration(color: color, borderRadius: borderRadius),
                    child: child),
                onTap: onTap)
        ));
  }
}