import 'package:mobx/mobx.dart';
import 'dart:developer';
import 'package:vesti_logger/logger/vlog.dart';

part 'vlogger_store.g.dart';

class VLoggerStore = _VLoggerStore with _$VLoggerStore;

abstract class _VLoggerStore with Store {
  final bool isEnabled;

  @observable
  String filter = "";

  @observable
  bool isDebug = false;

  @observable
  bool isError = false;

  @observable
  bool isInfo = false;

  @computed
  ObservableList<VLog> get filteredLogs => _logFiltered();

  @observable
  ObservableList<VLog> logs = ObservableList();

  _VLoggerStore(this.isEnabled);

  reset() {
    isDebug = false;
    isError = false;
    isInfo = false;
    filter = "";
  }

  debug({String? title, String? message}) {
    _log(title: title, message: message, type: VLogType.Debug);
  }

  error({String? title, String? message}) {
    _log(title: title, message: message, type: VLogType.Error);
  }

  info({String? title, String? message}) {
    _log(title: title, message: message, type: VLogType.Info);
  }

  _log({String? title, String? message, VLogType type = VLogType.Debug}) {
    if (isEnabled) {
      String updatedTitle = "$title\n" ?? "";
      log(type.separator +
          "\n" +
          updatedTitle +
          (message ?? "") +
          "\n" +
          type.separator);
      logs.add(VLog(title: title, message: message, type: type));
    }
  }

  ObservableList<VLog> _logFiltered() {
    List<VLog> filteredTypeLogs = _logTypeFiltered();
    List<VLog> filteredLogs = filteredTypeLogs.where((log) {
      return log.message?.toLowerCase().contains(filter) ?? false;
    }).toList();
    return ObservableList<VLog>.of(filteredLogs);
  }

  List<VLog> _logTypeFiltered() {
    List<VLog> filteredTypeLogs = [];
    if (isDebug == false && isError == false && isInfo == false) {
      filteredTypeLogs = logs.toList();
      return filteredTypeLogs;
    }
    if (isDebug)
      filteredTypeLogs.addAll(logs.where((log) => log.type == VLogType.Debug));

    if (isError)
      filteredTypeLogs.addAll(logs.where((log) => log.type == VLogType.Error));

    if (isInfo)
      filteredTypeLogs.addAll(logs.where((log) => log.type == VLogType.Info));

    return filteredTypeLogs;
  }
}